package cn.iocoder.yudao.module.configure.service.deviceenvtype;

import cn.iocoder.yudao.module.configure.controller.admin.deviceenvtype.vo.DeviceEnvTypeCreateReqVO;
import cn.iocoder.yudao.module.configure.controller.admin.deviceenvtype.vo.DeviceEnvTypeRespVO;
import cn.iocoder.yudao.module.configure.controller.admin.deviceenvtype.vo.DeviceEnvTypeUpdateReqVO;
import cn.iocoder.yudao.module.configure.dal.dataobject.deviceenvtype.DeviceEnvTypeDO;

import javax.validation.Valid;
import java.util.List;

/**
 * 小程序设备环境类型配置 Service 接口
 *
 * @author 芋道源码
 */
public interface DeviceEnvTypeService {

    /**
     * 创建小程序设备环境类型配置
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createDeviceEnvType(@Valid DeviceEnvTypeCreateReqVO createReqVO);

    /**
     * 更新小程序设备环境类型配置
     *
     * @param updateReqVO 更新信息
     */
    void updateDeviceEnvType(@Valid DeviceEnvTypeUpdateReqVO updateReqVO);

    /**
     * 删除小程序设备环境类型配置
     *
     * @param id 编号
     */
    void deleteDeviceEnvType(Long id);

    /**
     * 获得小程序设备环境类型配置
     *
     * @param id 编号
     * @return 小程序设备环境类型配置
     */
    DeviceEnvTypeDO getDeviceEnvType(Long id);

    /**
     * 批量创建小程序设备环境类型配置
     * @param createReqVOS 创建信息
     * @return 编号
     */
    List<Long> createDeviceEnvTypeBatch(@Valid List<DeviceEnvTypeCreateReqVO> createReqVOS);

    /**
     * 批量更新小程序设备环境类型配置
     * @param updateReqVOS 更新信息
     */
    void updateDeviceEnvTypeBatch(@Valid List<DeviceEnvTypeUpdateReqVO> updateReqVOS);

    /**
     * 批量删除小程序设备环境类型配置
     * @param ids 编号
     */
    void deleteDeviceEnvTypeBatch(List<Long> ids);

    /**
     * 根据地块编码获得环境类型配置
     * @param landId 地块编码
     * @return 环境类型配置
     */
    List<DeviceEnvTypeDO> getByLandId(String landId);

}
