package cn.iocoder.yudao.module.configure.service.materialgroup;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.configure.controller.admin.materialgroup.vo.MaterialGroupCreateReqVO;
import cn.iocoder.yudao.module.configure.controller.admin.materialgroup.vo.MaterialGroupPageReqVO;
import cn.iocoder.yudao.module.configure.controller.admin.materialgroup.vo.MaterialGroupUpdateReqVO;
import cn.iocoder.yudao.module.configure.convert.materialgroup.MaterialGroupConvert;
import cn.iocoder.yudao.module.configure.dal.dataobject.materialgroup.MaterialGroupDO;
import cn.iocoder.yudao.module.configure.dal.dataobject.materialmanagement.MaterialManagementDO;
import cn.iocoder.yudao.module.configure.dal.mysql.materialgroup.MaterialGroupMapper;
import cn.iocoder.yudao.module.configure.dal.mysql.materialmanagement.MaterialManagementMapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.configure.enums.ErrorCodeConstants.MATERIAL_GROUP_EXISTS_RELATED;
import static cn.iocoder.yudao.module.configure.enums.ErrorCodeConstants.MATERIAL_GROUP_NOT_EXISTS;

/**
 * 素材管理分组 Service 实现类
 *
 * @author 芋道源码
 */
@Service
@Validated
public class MaterialGroupServiceImpl implements MaterialGroupService {

    @Resource
    private MaterialGroupMapper materialGroupMapper;
    @Resource
    private MaterialManagementMapper materialManagementMapper;

    @Override
    public Long createMaterialGroup(MaterialGroupCreateReqVO createReqVO) {
        // 插入
        MaterialGroupDO materialGroup = MaterialGroupConvert.INSTANCE.convert(createReqVO);
        materialGroupMapper.insert(materialGroup);
        // 返回
        return materialGroup.getId();
    }

    @Override
    public void updateMaterialGroup(MaterialGroupUpdateReqVO updateReqVO) {
        // 校验存在
        validateMaterialGroupExists(updateReqVO.getId());
        // 更新
        MaterialGroupDO updateObj = MaterialGroupConvert.INSTANCE.convert(updateReqVO);
        materialGroupMapper.updateById(updateObj);
    }

    @Override
    public void deleteMaterialGroup(Long id) {
        // 校验存在
        validateMaterialGroupExists(id);
        // 校验是否存在关联素材
        if (
                materialManagementMapper.exists(new LambdaQueryWrapperX<MaterialManagementDO>()
                        .eq(MaterialManagementDO::getGroupId, id)
                        .eq(MaterialManagementDO::getDeleted, 0)
                )
        ) {
            throw exception(MATERIAL_GROUP_EXISTS_RELATED);
        }
        // 删除
        materialGroupMapper.update(null, new LambdaUpdateWrapper<MaterialGroupDO>()
                .set(MaterialGroupDO::getDeleted, 1)
                .eq(MaterialGroupDO::getId, id)
        );
    }

    private void validateMaterialGroupExists(Long id) {
        if (materialGroupMapper.selectById(id) == null) {
            throw exception(MATERIAL_GROUP_NOT_EXISTS);
        }
    }

    @Override
    public MaterialGroupDO getMaterialGroup(Long id) {
        return materialGroupMapper.selectById(id);
    }

    @Override
    public List<MaterialGroupDO> getMaterialGroupListByids(Collection<Long> ids) {
        return materialGroupMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<MaterialGroupDO> getMaterialGroupPage(MaterialGroupPageReqVO pageReqVO) {
        return materialGroupMapper.selectPage(pageReqVO);
    }

    @Override
    public List<MaterialGroupDO> getMaterialGroupList(MaterialGroupPageReqVO pageReqVO) {
        return materialGroupMapper.selectList(pageReqVO);
    }

}
