package cn.iocoder.yudao.module.configure.service.tracetemplate;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.test.core.ut.BaseDbUnitTest;
import cn.iocoder.yudao.module.configure.controller.admin.tracetemplate.vo.TraceTemplateCreateReqVO;
import cn.iocoder.yudao.module.configure.controller.admin.tracetemplate.vo.TraceTemplatePageReqVO;
import cn.iocoder.yudao.module.configure.controller.admin.tracetemplate.vo.TraceTemplateRespVO;
import cn.iocoder.yudao.module.configure.controller.admin.tracetemplate.vo.TraceTemplateUpdateReqVO;
import cn.iocoder.yudao.module.configure.dal.dataobject.tracetemplate.TraceTemplateDO;
import cn.iocoder.yudao.module.configure.dal.mysql.tracetemplate.TraceTemplateMapper;
import cn.iocoder.yudao.module.configure.enums.ErrorCodeConstants;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.context.annotation.Import;

import javax.annotation.Resource;

import static cn.iocoder.yudao.framework.common.util.object.ObjectUtils.cloneIgnoreId;
import static cn.iocoder.yudao.framework.test.core.util.AssertUtils.assertPojoEquals;
import static cn.iocoder.yudao.framework.test.core.util.AssertUtils.assertServiceException;
import static cn.iocoder.yudao.framework.test.core.util.RandomUtils.randomLongId;
import static cn.iocoder.yudao.framework.test.core.util.RandomUtils.randomPojo;
import static org.junit.jupiter.api.Assertions.*;

/**
* {@link TraceTemplateServiceImpl} 的单元测试类
*
* @author 芋道源码
*/
@Import(TraceTemplateServiceImpl.class)
public class TraceTemplateServiceImplTest extends BaseDbUnitTest {

    @Resource
    private TraceTemplateServiceImpl traceTemplateService;

    @Resource
    private TraceTemplateMapper traceTemplateMapper;

    @Test
    public void testCreateTraceTemplate_success() {
        // 准备参数
        TraceTemplateCreateReqVO reqVO = randomPojo(TraceTemplateCreateReqVO.class);

        // 调用
        Long traceTemplateId = traceTemplateService.createTraceTemplate(reqVO);
        // 断言
        assertNotNull(traceTemplateId);
        // 校验记录的属性是否正确
        TraceTemplateDO traceTemplate = traceTemplateMapper.selectById(traceTemplateId);
        assertPojoEquals(reqVO, traceTemplate);
    }

    @Test
    public void testUpdateTraceTemplate_success() {
        // mock 数据
        TraceTemplateDO dbTraceTemplate = randomPojo(TraceTemplateDO.class);
        traceTemplateMapper.insert(dbTraceTemplate);// @Sql: 先插入出一条存在的数据
        // 准备参数
        TraceTemplateUpdateReqVO reqVO = randomPojo(TraceTemplateUpdateReqVO.class, o -> {
            o.setId(dbTraceTemplate.getId()); // 设置更新的 ID
        });

        // 调用
        traceTemplateService.updateTraceTemplate(reqVO);
        // 校验是否更新正确
        TraceTemplateDO traceTemplate = traceTemplateMapper.selectById(reqVO.getId()); // 获取最新的
        assertPojoEquals(reqVO, traceTemplate);
    }

    @Test
    public void testUpdateTraceTemplate_notExists() {
        // 准备参数
        TraceTemplateUpdateReqVO reqVO = randomPojo(TraceTemplateUpdateReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> traceTemplateService.updateTraceTemplate(reqVO), ErrorCodeConstants.TRACE_TEMPLATE_NOT_EXISTS);
    }

    @Test
    public void testDeleteTraceTemplate_success() {
        // mock 数据
        TraceTemplateDO dbTraceTemplate = randomPojo(TraceTemplateDO.class);
        traceTemplateMapper.insert(dbTraceTemplate);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbTraceTemplate.getId();

        // 调用
        traceTemplateService.deleteTraceTemplate(id);
       // 校验数据不存在了
       assertNull(traceTemplateMapper.selectById(id));
    }

    @Test
    public void testDeleteTraceTemplate_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> traceTemplateService.deleteTraceTemplate(id), ErrorCodeConstants.TRACE_TEMPLATE_NOT_EXISTS);
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetTraceTemplatePage() {
       // mock 数据
       TraceTemplateDO dbTraceTemplate = randomPojo(TraceTemplateDO.class, o -> { // 等会查询到
           o.setTemplateId(null);
           o.setTemplateName(null);
           o.setGoodsId(null);
           o.setGoodsName(null);
           o.setBelongBusinessId(null);
           o.setBelongBusinessName(null);
           o.setCreateTime(null);
       });
       traceTemplateMapper.insert(dbTraceTemplate);
       // 测试 templateId 不匹配
       traceTemplateMapper.insert(cloneIgnoreId(dbTraceTemplate, o -> o.setTemplateId(null)));
       // 测试 templateName 不匹配
       traceTemplateMapper.insert(cloneIgnoreId(dbTraceTemplate, o -> o.setTemplateName(null)));
       // 测试 goodsId 不匹配
       traceTemplateMapper.insert(cloneIgnoreId(dbTraceTemplate, o -> o.setGoodsId(null)));
       // 测试 goodsName 不匹配
       traceTemplateMapper.insert(cloneIgnoreId(dbTraceTemplate, o -> o.setGoodsName(null)));
       // 测试 belongBusinessId 不匹配
       traceTemplateMapper.insert(cloneIgnoreId(dbTraceTemplate, o -> o.setBelongBusinessId(null)));
       // 测试 belongBusinessName 不匹配
       traceTemplateMapper.insert(cloneIgnoreId(dbTraceTemplate, o -> o.setBelongBusinessName(null)));
       // 测试 createTime 不匹配
       traceTemplateMapper.insert(cloneIgnoreId(dbTraceTemplate, o -> o.setCreateTime(null)));
       // 准备参数
       TraceTemplatePageReqVO reqVO = new TraceTemplatePageReqVO();
       reqVO.setTemplateName(null);
       reqVO.setGoodsName(null);
       reqVO.setBelongBusinessName(null);

       // 调用
        PageResult<TraceTemplateRespVO> pageResult = traceTemplateService.getTraceTemplatePage(reqVO);
       // 断言
       assertEquals(1, pageResult.getTotal());
       assertEquals(1, pageResult.getList().size());
       assertPojoEquals(dbTraceTemplate, pageResult.getList().get(0));
    }

}
