package cn.iocoder.yudao.module.configure.service.unitemplateitem;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.mock.mockito.MockBean;

import javax.annotation.Resource;

import cn.iocoder.yudao.framework.test.core.ut.BaseDbUnitTest;

import cn.iocoder.yudao.module.configure.controller.admin.unitemplateitem.vo.*;
import cn.iocoder.yudao.module.configure.dal.dataobject.unitemplateitem.UniTemplateItemDO;
import cn.iocoder.yudao.module.configure.dal.mysql.unitemplateitem.UniTemplateItemMapper;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import javax.annotation.Resource;
import org.springframework.context.annotation.Import;
import java.util.*;
import java.time.LocalDateTime;

import static cn.hutool.core.util.RandomUtil.*;
import static cn.iocoder.yudao.module.configure.enums.ErrorCodeConstants.*;
import static cn.iocoder.yudao.framework.test.core.util.AssertUtils.*;
import static cn.iocoder.yudao.framework.test.core.util.RandomUtils.*;
import static cn.iocoder.yudao.framework.common.util.object.ObjectUtils.*;
import static cn.iocoder.yudao.framework.common.util.date.DateUtils.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

/**
* {@link UniTemplateItemServiceImpl} 的单元测试类
*
* @author 芋道源码
*/
@Import(UniTemplateItemServiceImpl.class)
public class UniTemplateItemServiceImplTest extends BaseDbUnitTest {

    @Resource
    private UniTemplateItemServiceImpl uniTemplateItemService;

    @Resource
    private UniTemplateItemMapper uniTemplateItemMapper;

    @Test
    public void testCreateUniTemplateItem_success() {
        // 准备参数
        UniTemplateItemCreateReqVO reqVO = randomPojo(UniTemplateItemCreateReqVO.class);

        // 调用
        Long uniTemplateItemId = uniTemplateItemService.createUniTemplateItem(reqVO);
        // 断言
        assertNotNull(uniTemplateItemId);
        // 校验记录的属性是否正确
        UniTemplateItemDO uniTemplateItem = uniTemplateItemMapper.selectById(uniTemplateItemId);
        assertPojoEquals(reqVO, uniTemplateItem);
    }

    @Test
    public void testUpdateUniTemplateItem_success() {
        // mock 数据
        UniTemplateItemDO dbUniTemplateItem = randomPojo(UniTemplateItemDO.class);
        uniTemplateItemMapper.insert(dbUniTemplateItem);// @Sql: 先插入出一条存在的数据
        // 准备参数
        UniTemplateItemUpdateReqVO reqVO = randomPojo(UniTemplateItemUpdateReqVO.class, o -> {
            o.setId(dbUniTemplateItem.getId()); // 设置更新的 ID
        });

        // 调用
        uniTemplateItemService.updateUniTemplateItem(reqVO);
        // 校验是否更新正确
        UniTemplateItemDO uniTemplateItem = uniTemplateItemMapper.selectById(reqVO.getId()); // 获取最新的
        assertPojoEquals(reqVO, uniTemplateItem);
    }

    @Test
    public void testUpdateUniTemplateItem_notExists() {
        // 准备参数
        UniTemplateItemUpdateReqVO reqVO = randomPojo(UniTemplateItemUpdateReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> uniTemplateItemService.updateUniTemplateItem(reqVO), UNI_TEMPLATE_ITEM_NOT_EXISTS);
    }

    @Test
    public void testDeleteUniTemplateItem_success() {
        // mock 数据
        UniTemplateItemDO dbUniTemplateItem = randomPojo(UniTemplateItemDO.class);
        uniTemplateItemMapper.insert(dbUniTemplateItem);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbUniTemplateItem.getId();

        // 调用
        uniTemplateItemService.deleteUniTemplateItem(id);
       // 校验数据不存在了
       assertNull(uniTemplateItemMapper.selectById(id));
    }

    @Test
    public void testDeleteUniTemplateItem_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> uniTemplateItemService.deleteUniTemplateItem(id), UNI_TEMPLATE_ITEM_NOT_EXISTS);
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetUniTemplateItemPage() {
       // mock 数据
       UniTemplateItemDO dbUniTemplateItem = randomPojo(UniTemplateItemDO.class, o -> { // 等会查询到
           o.setTemplateId(null);
           o.setIconUrl(null);
           o.setPosition(null);
           o.setWidth(null);
           o.setHeight(null);
           o.setBelongBusinessId(null);
           o.setBelongBusinessName(null);
           o.setCreateTime(null);
       });
       uniTemplateItemMapper.insert(dbUniTemplateItem);
       // 测试 templateId 不匹配
       uniTemplateItemMapper.insert(cloneIgnoreId(dbUniTemplateItem, o -> o.setTemplateId(null)));
       // 测试 iconUrl 不匹配
       uniTemplateItemMapper.insert(cloneIgnoreId(dbUniTemplateItem, o -> o.setIconUrl(null)));
       // 测试 position 不匹配
       uniTemplateItemMapper.insert(cloneIgnoreId(dbUniTemplateItem, o -> o.setPosition(null)));
       // 测试 width 不匹配
       uniTemplateItemMapper.insert(cloneIgnoreId(dbUniTemplateItem, o -> o.setWidth(null)));
       // 测试 height 不匹配
       uniTemplateItemMapper.insert(cloneIgnoreId(dbUniTemplateItem, o -> o.setHeight(null)));
       // 测试 belongBusinessId 不匹配
       uniTemplateItemMapper.insert(cloneIgnoreId(dbUniTemplateItem, o -> o.setBelongBusinessId(null)));
       // 测试 belongBusinessName 不匹配
       uniTemplateItemMapper.insert(cloneIgnoreId(dbUniTemplateItem, o -> o.setBelongBusinessName(null)));
       // 测试 createTime 不匹配
       uniTemplateItemMapper.insert(cloneIgnoreId(dbUniTemplateItem, o -> o.setCreateTime(null)));
       // 准备参数
       UniTemplateItemPageReqVO reqVO = new UniTemplateItemPageReqVO();
       reqVO.setTemplateId(null);
       reqVO.setIconUrl(null);
       reqVO.setPosition(null);
       reqVO.setWidth(null);
       reqVO.setHeight(null);
       reqVO.setBelongBusinessId(null);
       reqVO.setBelongBusinessName(null);
       reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));

       // 调用
       PageResult<UniTemplateItemDO> pageResult = uniTemplateItemService.getUniTemplateItemPage(reqVO);
       // 断言
       assertEquals(1, pageResult.getTotal());
       assertEquals(1, pageResult.getList().size());
       assertPojoEquals(dbUniTemplateItem, pageResult.getList().get(0));
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetUniTemplateItemList() {
       // mock 数据
       UniTemplateItemDO dbUniTemplateItem = randomPojo(UniTemplateItemDO.class, o -> { // 等会查询到
           o.setTemplateId(null);
           o.setIconUrl(null);
           o.setPosition(null);
           o.setWidth(null);
           o.setHeight(null);
           o.setBelongBusinessId(null);
           o.setBelongBusinessName(null);
           o.setCreateTime(null);
       });
       uniTemplateItemMapper.insert(dbUniTemplateItem);
       // 测试 templateId 不匹配
       uniTemplateItemMapper.insert(cloneIgnoreId(dbUniTemplateItem, o -> o.setTemplateId(null)));
       // 测试 iconUrl 不匹配
       uniTemplateItemMapper.insert(cloneIgnoreId(dbUniTemplateItem, o -> o.setIconUrl(null)));
       // 测试 position 不匹配
       uniTemplateItemMapper.insert(cloneIgnoreId(dbUniTemplateItem, o -> o.setPosition(null)));
       // 测试 width 不匹配
       uniTemplateItemMapper.insert(cloneIgnoreId(dbUniTemplateItem, o -> o.setWidth(null)));
       // 测试 height 不匹配
       uniTemplateItemMapper.insert(cloneIgnoreId(dbUniTemplateItem, o -> o.setHeight(null)));
       // 测试 belongBusinessId 不匹配
       uniTemplateItemMapper.insert(cloneIgnoreId(dbUniTemplateItem, o -> o.setBelongBusinessId(null)));
       // 测试 belongBusinessName 不匹配
       uniTemplateItemMapper.insert(cloneIgnoreId(dbUniTemplateItem, o -> o.setBelongBusinessName(null)));
       // 测试 createTime 不匹配
       uniTemplateItemMapper.insert(cloneIgnoreId(dbUniTemplateItem, o -> o.setCreateTime(null)));
       // 准备参数
       UniTemplateItemExportReqVO reqVO = new UniTemplateItemExportReqVO();
       reqVO.setTemplateId(null);
       reqVO.setIconUrl(null);
       reqVO.setPosition(null);
       reqVO.setWidth(null);
       reqVO.setHeight(null);
       reqVO.setBelongBusinessId(null);
       reqVO.setBelongBusinessName(null);
       reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));

       // 调用
       List<UniTemplateItemDO> list = uniTemplateItemService.getUniTemplateItemList(reqVO);
       // 断言
       assertEquals(1, list.size());
       assertPojoEquals(dbUniTemplateItem, list.get(0));
    }

}
