package cn.iocoder.yudao.module.identity.service.code;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.module.identity.controller.admin.code.vo.*;
import cn.iocoder.yudao.module.identity.dal.dataobject.code.CodeDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

/**
 * 标识码 Service 接口
 *
 * @author 芋道源码
 */
public interface CodeService {

    /**
     * 创建标识码组
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long[] createCode(@Valid CodeCreateReqVO createReqVO);

    /**
     * 更新标识码
     *
     * @param updateReqVO 更新信息
     */
    void updateCode(@Valid CodeUpdateReqVO updateReqVO);

    /**
     * 删除标识码
     *
     * @param id 编号
     */
    void deleteCode(Long id);

    /**
     * 获得标识码
     *
     * @param id 编号
     * @return 标识码
     */
    CodeDO getCode(Long id);

    /**
     * 获得标识码分页
     *
     * @param pageReqVO 分页查询
     * @return 标识码分页
     */
    PageResult<CodeDO> getCodePage(CodePageReqVO pageReqVO);

    /**
     * 获得标识码列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 标识码列表
     */
    List<CodeDO> getCodeList(CodeExportReqVO exportReqVO);

    /**
     * 查询所有标识码列表
     * @return
     */
    List<CodeDO> getCodeList();

    /**
     * 根据指定的generateRecordId获取所有标识码列表
     * @param generateRecordId
     * @return
     */
    List<CodeDO> getCodesByGenerateRecordId(String generateRecordId);

    /**
     * 根据商户ID和identityCode查询对应的标识码信息
     * @param identityCode
     * @return
     */
    List<CodeDO> getCodeByGenerateRecordIdAndIdentityCode(String generateRecordId,String identityCode);

    /**
     * 为小程序获得二维码数据
     * @param identityCode 二维码
     * @return 二维码
     */
    CodeDO getCodeForApp(String identityCode);

    /**
     * 根据指定的生码记录id获取批次id和商品昵称
     * @param generateRecordId
     * @return
     */
    CodeDO getCodeDOInfoByGenerateRecordId(String generateRecordId);
}
