package cn.iocoder.yudao.module.product.service.deviceinfo;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.product.controller.admin.deviceinfo.vo.*;

import javax.validation.Valid;
import java.util.List;

/**
 * 设备信息 Service 接口
 *
 * @author 芋道源码
 */
public interface DeviceInfoService {

    /**
     * 创建设备信息
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createDeviceInfo(@Valid DeviceInfoCreateReqVO createReqVO);

    /**
     * 更新设备信息
     *
     * @param updateReqVO 更新信息
     */
    void updateDeviceInfo(@Valid DeviceInfoUpdateReqVO updateReqVO);

    /**
     * 删除设备信息
     *
     * @param id 编号
     */
    void deleteDeviceInfo(Long id);

    /**
     * 获得设备信息
     *
     * @param id 编号
     * @return 设备信息
     */
    DeviceInfoRespVO getDeviceInfo(Long id);

    /**
     * 获得设备信息分页
     *
     * @param pageReqVO 分页查询
     * @return 设备信息分页
     */
    PageResult<DeviceInfoRespVO> getDeviceInfoPage(DeviceInfoPageReqVO pageReqVO);

    /**
     * 更改开关状态
     *
     * @param switchOn 是否打开开关
     */
    void changeSwitch(Long id, Boolean switchOn);

    /**
     * 更改是否启用
     *
     * @param enabled 是否启用
     */
    void changeEnabled(Long id, Boolean enabled);

    /**
     * 获取监控播放地址
     *
     * @param protocol          类型 ws(http),wss(https)
     */
    MonitorUrLRespVO getMonitorPreviewURL(String monitorCode, String protocol);

    /**
     * 获取监控播放地址
     *
     * @param landId 地块/鱼塘/牧场编码
     * @param protocol   类型 ws(http),wss(https)
     */
    List<MonitorUrLRespVO> getMonitorPreviewURLByLandId(String landId, String protocol);
}
