package cn.iocoder.yudao.module.product.service.envtype;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.module.product.controller.admin.envtype.vo.*;
import cn.iocoder.yudao.module.product.dal.dataobject.envtype.EnvTypeDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

/**
 * 环境类型 Service 接口
 *
 * @author 芋道源码
 */
public interface EnvTypeService {

    /**
     * 创建环境类型
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createEnvType(@Valid EnvTypeCreateReqVO createReqVO);

    /**
     * 更新环境类型
     *
     * @param updateReqVO 更新信息
     */
    void updateEnvType(@Valid EnvTypeUpdateReqVO updateReqVO);

    /**
     * 删除环境类型
     *
     * @param id 编号
     */
    void deleteEnvType(Long id);

    /**
     * 获得环境类型
     *
     * @param id 编号
     * @return 环境类型
     */
    EnvTypeDO getEnvType(Long id);

    /**
     * 获得环境类型分页
     *
     * @param pageReqVO 分页查询
     * @return 环境类型分页
     */
    PageResult<EnvTypeDO> getEnvTypePage(EnvTypePageReqVO pageReqVO);

    /**
     * 获得所有环境类型
     * @return 环境类型
     */
    List<EnvTypeDO> getAllEnvType();

    /**
     * 根据商户ID获得环境类型
     * @param businessInfoId 商户ID
     * @return 环境类型
     */
    List<EnvTypeDO> getAvailableEnvTypeByBusinessId(Long businessInfoId);

    /**
     * 根据商户ID获得环境类型配置（根据地块编码分组
     * @param businessInfoId 商户ID
     * @return 环境类型配置
     */
    List<EnvTypeDeviceConfigRespVO> getEnvTypeConfigGroupByLandId(Long businessInfoId);

}
