package cn.iocoder.yudao.module.product.service.envtype;

import cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.product.controller.admin.envtype.vo.EnvTypeDeviceConfigRespVO;
import cn.iocoder.yudao.module.product.controller.admin.envtype.vo.EnvTypeCreateReqVO;
import cn.iocoder.yudao.module.product.controller.admin.envtype.vo.EnvTypePageReqVO;
import cn.iocoder.yudao.module.product.controller.admin.envtype.vo.EnvTypeUpdateReqVO;
import cn.iocoder.yudao.module.product.convert.envtype.EnvTypeConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.deviceinfo.DeviceInfoDO;
import cn.iocoder.yudao.module.product.dal.dataobject.envtype.EnvTypeDO;
import cn.iocoder.yudao.module.product.dal.mysql.deviceinfo.DeviceInfoMapper;
import cn.iocoder.yudao.module.product.dal.mysql.envtype.EnvTypeMapper;
import cn.iocoder.yudao.module.product.enums.ErrorCodeConstants;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.List;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.ENV_TYPE_NOT_EXISTS;

/**
 * 环境类型 Service 实现类
 *
 * @author 芋道源码
 */
@Service
@Validated
public class EnvTypeServiceImpl implements EnvTypeService {

    @Resource
    private EnvTypeMapper envTypeMapper;
    @Resource
    private DeviceInfoMapper deviceInfoMapper;

    @Override
    public Long createEnvType(EnvTypeCreateReqVO createReqVO) {
        EnvTypeDO envType = EnvTypeConvert.INSTANCE.convert(createReqVO);
        envTypeMapper.insert(envType);

        // 返回
        return envType.getId();
    }

    @Override
    public void updateEnvType(EnvTypeUpdateReqVO updateReqVO) {
        // 校验存在
        validateEnvTypeExists(updateReqVO.getId());
        // 更新
        EnvTypeDO updateObj = EnvTypeConvert.INSTANCE.convert(updateReqVO);
        envTypeMapper.updateById(updateObj);
    }

    @Override
    public void deleteEnvType(Long id) {
        // 校验存在
        validateEnvTypeExists(id);
        // 判断是否有设备引用到该控制类型
        List<DeviceInfoDO> deviceInfos = deviceInfoMapper.selectList(new LambdaQueryWrapperX<DeviceInfoDO>().eq(DeviceInfoDO::getEnvType, id));
        if(deviceInfos.isEmpty()) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.CAN_NOT_DELETE_WHEN_EXIST_DEVICE_INFO);
        }
        // 删除
        envTypeMapper.deleteById(id);
    }

    private void validateEnvTypeExists(Long id) {
        if (envTypeMapper.selectById(id) == null) {
            throw exception(ENV_TYPE_NOT_EXISTS);
        }
    }

    @Override
    public EnvTypeDO getEnvType(Long id) {
        return envTypeMapper.selectById(id);
    }

    @Override
    public PageResult<EnvTypeDO> getEnvTypePage(EnvTypePageReqVO pageReqVO) {
        return envTypeMapper.selectPage(pageReqVO);
    }

    @Override
    public List<EnvTypeDO> getAllEnvType() {
        return envTypeMapper.selectList();
    }

    @Override
    public List<EnvTypeDO> getAvailableEnvTypeByBusinessId(Long businessInfoId) {
        return envTypeMapper.getAvailableEnvType(businessInfoId);
    }

    @Override
    public List<EnvTypeDeviceConfigRespVO> getEnvTypeConfigGroupByLandId(Long businessInfoId) {
        return envTypeMapper.getEnvTypeConfigGroupByLandId(businessInfoId);
    }
}
