package cn.iocoder.yudao.module.product.service.harvestbatch;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.product.controller.admin.harvestbatch.vo.*;
import cn.iocoder.yudao.module.product.dal.dataobject.harvestbatch.HarvestBatchDO;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * 采收批次 Service 接口
 *
 * @author 芋道源码
 */
public interface HarvestBatchService {

    /**
     * 获得采收批次
     *
     * @param id 编号
     * @return 采收批次
     */
    HarvestBatchRespVO getHarvestBatchDetail(Long id);

    List<String> getAllBatchCode();

    /**
     * 获得采收批次分页
     *
     * @param pageReqVO 分页查询
     * @return 采收批次分页
     */
    PageResult<HarvestBatchDO> getHarvestBatchPage(HarvestBatchPageReqVO pageReqVO);

    /**
     * 获得采收批次编号
     * @param count 数量
     * @return 编码
     */
    List<String> getHarverstBatchIdList(Integer count);

    /**
     * 获得商品信息配置 - 溯源环节相关
     * @param id id
     */
    GoodsConfigRespVO getGoodsConfig(Long id);

    /**
     * 创建商品信息配置 - 溯源环节相关
     * @param createReqVO 入参
     * @return 是否成功
     */
    Boolean createGoodsConfig(@Valid GoodsConfigSaveReqVO createReqVO);

    /**
     * 更新商品信息配置 - 溯源环节相关
     * @param updateReqVO 入参
     */
    void updateGoodsConfig(@Valid GoodsConfigSaveReqVO updateReqVO);

    /**
     * 获得环节配置 - 溯源环节相关
     * @param id id
     * @return 环节配置
     */
    LinkConfigRespVO getLinkConfig(Long id);

    /**
     * 更新环节信息配置 - 溯源环节相关
     * @param updateReqVO 入参
     */
    void updateLinkConfig(@Valid LinkConfigSaveReqVO updateReqVO);

    /**
     * 获得农事活动配置 - 溯源环节相关
     * @param id id
     * @return 农事配置
     */
    FarmingConfigRespVO getFarmingConfig(Long id);

    /**
     * 更新农事活动配置 - 溯源环节相关
     * @param updateReqVO 入参
     */
    void updateFarmingConfig(@Valid FarmingConfigSaveReqVO updateReqVO);

    /**
     * 根据商品编码集合获取该集合对应的批次编码集合
     * @param goodsIds 商品编码集合
     * @return 该集合对应的批次编码集合
     */
    Map<String, List<String>> getBatchIdsByGoodsId(List<String> goodsIds, Long businessId, Integer blockState);

    /**
     * 根据商品批次id获取批次所有信息
     * @param harvestBatchId 批次编码
     * @return 批次
     */
    HarvestBatchDO getHarvestBatchByHarvesId(String harvestBatchId);

    /**
     * 条件查询
     * 根据商户ID，已上链，俩个条件查询出所有采收批次id
     * @param belongBusinessId 商户id
     * @param blockState 链状态
     * @return 批次
     */
    List<HarvestBatchDO> getHarvestBatchByBusinessId(String belongBusinessId, Integer blockState);

    /**
     * 获得小程序预览数据
     * @param harvestBatchId 批次编码
     * @return 小程序预览数据
     */
    PreviewRespVO getPreviewData(String harvestBatchId, Long languageTypeId);

    /**
     * 获得小程序需要的作业信息
     * @param chainOperateId 区块编码
     * @return 作业信息
     */
    List<BlockCertificateSaveReqVO> gatJobData(String chainOperateId);

    /**
     * 根据采收批次ID获取采收批次信息
     * @param harvestBatchId 批次编码
     * @return 批次
     */
    HarvestBatchDO getHarvestBatchById(String harvestBatchId);

    /**
     * 更新链状态
     * @param id id
     * @param state 状态
     */
    void updateChainStatus(Long id, Integer state);

    /**
     * 获得采收批次配置信息
     * @param id id
     * @return 配置信息
     */
    HarvestBatchConfigRespVO getHarvestBatchConfig(Long id, Long languageTypeId);

    /**
     * 更新采收批次配置
     * @param saveReqVO 入参
     */
    void updateHarvestBatchConfig(@Valid HarvestBatchConfigSaveReqVO saveReqVO);

    void initHarvestBatchConfig();

}
