package cn.iocoder.yudao.module.product.service.envtype;

import cn.iocoder.yudao.framework.test.core.ut.BaseDbUnitTest;
import cn.iocoder.yudao.module.product.controller.admin.envtype.vo.EnvTypeCreateReqVO;
import cn.iocoder.yudao.module.product.controller.admin.envtype.vo.EnvTypeUpdateReqVO;
import cn.iocoder.yudao.module.product.dal.dataobject.envtype.EnvTypeDO;
import cn.iocoder.yudao.module.product.dal.mysql.envtype.EnvTypeMapper;
import org.junit.jupiter.api.Test;
import org.springframework.context.annotation.Import;

import javax.annotation.Resource;

import static cn.iocoder.yudao.framework.test.core.util.AssertUtils.assertPojoEquals;
import static cn.iocoder.yudao.framework.test.core.util.AssertUtils.assertServiceException;
import static cn.iocoder.yudao.framework.test.core.util.RandomUtils.randomLongId;
import static cn.iocoder.yudao.framework.test.core.util.RandomUtils.randomPojo;
import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.ENV_TYPE_NOT_EXISTS;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertNull;

/**
* {@link EnvTypeServiceImpl} 的单元测试类
*
* @author 芋道源码
*/
@Import(EnvTypeServiceImpl.class)
public class EnvTypeServiceImplTest extends BaseDbUnitTest {

    @Resource
    private EnvTypeServiceImpl envTypeService;

    @Resource
    private EnvTypeMapper envTypeMapper;

    @Test
    public void testCreateEnvType_success() {
        // 准备参数
        EnvTypeCreateReqVO reqVO = randomPojo(EnvTypeCreateReqVO.class);

        // 调用
        Long envTypeId = envTypeService.createEnvType(reqVO);
        // 断言
        assertNotNull(envTypeId);
        // 校验记录的属性是否正确
        EnvTypeDO envType = envTypeMapper.selectById(envTypeId);
        assertPojoEquals(reqVO, envType);
    }

    @Test
    public void testUpdateEnvType_success() {
        // mock 数据
        EnvTypeDO dbEnvType = randomPojo(EnvTypeDO.class);
        envTypeMapper.insert(dbEnvType);// @Sql: 先插入出一条存在的数据
        // 准备参数
        EnvTypeUpdateReqVO reqVO = randomPojo(EnvTypeUpdateReqVO.class, o -> {
            o.setId(dbEnvType.getId()); // 设置更新的 ID
        });

        // 调用
        envTypeService.updateEnvType(reqVO);
        // 校验是否更新正确
        EnvTypeDO envType = envTypeMapper.selectById(reqVO.getId()); // 获取最新的
        assertPojoEquals(reqVO, envType);
    }

    @Test
    public void testUpdateEnvType_notExists() {
        // 准备参数
        EnvTypeUpdateReqVO reqVO = randomPojo(EnvTypeUpdateReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> envTypeService.updateEnvType(reqVO), ENV_TYPE_NOT_EXISTS);
    }

    @Test
    public void testDeleteEnvType_success() {
        // mock 数据
        EnvTypeDO dbEnvType = randomPojo(EnvTypeDO.class);
        envTypeMapper.insert(dbEnvType);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbEnvType.getId();

        // 调用
        envTypeService.deleteEnvType(id);
       // 校验数据不存在了
       assertNull(envTypeMapper.selectById(id));
    }

    @Test
    public void testDeleteEnvType_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> envTypeService.deleteEnvType(id), ENV_TYPE_NOT_EXISTS);
    }

}
